% Compute figure/ground masks (top-down and combined) for all object detections
% in the image from their member poselet activations and associated member 
% pixels.
%
% [mask mask_fg] = obj_mask_dets_fg( ...
%    obj_bounds, poselet_hits, fg_masks, im_size, obj_px_inds, obj_px_vals)
%
% Input:
%    obj_bounds      - predicted object bounds and member poselets
%    poselet_hits    - poselets detected in the image
%    fg_masks        - learned poselet figure/ground masks
%    [sx sy]         - image size
%    obj_px_inds     - cell array containing indices of object pixels
%    obj_px_vals     - figure/ground ordering on object pixels
% 
% Output:
%    mask            - object mask created using top-down poselet detections
%    mask_fg         - reweighted object mask using both poselets and pixels
function [mask mask_fg] = obj_mask_dets_fg( ...
   obj_bounds, poselet_hits, fg_masks, im_size, obj_px_inds, obj_px_vals)
   % initialize masks
   mask    = zeros(im_size);
   mask_fg = zeros(im_size);
   % combine masks for all objects
   for n = 1:(obj_bounds.size)
      % get poselets used in object
      poselets_obj = poselet_hits.select(obj_bounds.src_idx{n});
      % compute masks for object
      mask_obj = obj_mask_poselets(poselets_obj, fg_masks, im_size);
      mask_fg(obj_px_inds{n}) = obj_px_vals{n};
      % combine masks
      mask = max(mask, mask_obj);
   end
   % warp top-down mask onto pixel membership mask
   mask_fg = normalize_hist(mask, mask_fg);
end
