% Compute top-down figure/ground mask for a single detected object by weighting
% its member poselet activations.
%
% mask = obj_mask_poselets(poselet_hits, fg_masks, [sx sy])
%
% Input:
%    poselet_hits    - poselets belonging to detected object
%    fg_masks        - learned poselet figure/ground masks
%    [sx sy]         - image size
%
% Output:
%    mask            - object mask created using top-down poselet detections
function mask = obj_mask_poselets(poselet_hits, fg_masks, im_size)
   % extract image size
   sx = im_size(1);
   sy = im_size(2);
   % compute poselet mask bounds
   xmin = 1; xmax = sx;
   ymin = 1; ymax = sy;
   for n = 1:(poselet_hits.size)
      bounds = round(double(poselet_hits.bounds(n,:)));
      ymin = min(ymin,bounds(1)); ymax = max(ymax,bounds(1)+bounds(3)-1);
      xmin = min(xmin,bounds(2)); xmax = max(xmax,bounds(2)+bounds(4)-1);
   end
   % calculate border size
   border_x = max(1-xmin, xmax-sx);
   border_y = max(1-ymin, ymax-sy);
   % initialize figure/ground image
   fg = zeros([sx+2.*border_x sy+2.*border_y]);
   wt = zeros([sx+2.*border_x sy+2.*border_y]);
   % place down f/g mask of each poselet weighted by score
   for n = 1:(poselet_hits.size)
      % get mask
      bounds = round(double(poselet_hits.bounds(n,:)));
      id = double(poselet_hits.poselet_id(n));
      fg_mask = fg_masks{id};
      w = poselet_hits.score(n);
      % resize
      ys = bounds(1); ylen = bounds(3); ye = ys+ylen-1;
      xs = bounds(2); xlen = bounds(4); xe = xs+xlen-1;
      fg_mask = imresize(fg_mask,[xlen ylen],'bilinear');
      % add to fg image
      fg(border_x+(xs:xe),border_y+(ys:ye)) = ...
         fg(border_x+(xs:xe),border_y+(ys:ye)) +  w.*fg_mask;
      wt(border_x+(xs:xe),border_y+(ys:ye)) = ...
         wt(border_x+(xs:xe),border_y+(ys:ye)) +  ones(size(fg_mask));
   end
   % clip away border
   fg = fg((border_x+1):(border_x+sx),(border_y+1):(border_y+sy));
   wt = wt((border_x+1):(border_x+sx),(border_y+1):(border_y+sy));
   % return mask
   mask = fg./(wt + eps);
end
