% Solve constrained Angular Embedding problem.
%
% [evecs evals] = ae_constrained(C, Theta, U, nv, opts)
%
% Input:
%    C      - ne x ne symmetric confidence matrix
%    Theta  - ne x ne skew-symmetric ordering matrix
%    U      - ne x nc constraint matrix
%    nv     - number of eigenvectors desired (use default if not specified)
%    opts   - eigensolver options (use defaults if not specified)
%
% Output:
%    evecs  - ne x nv eigenvectors
%    evals  - nv x 1  eigenvalues
function [evecs evals] = ae_constrained(C, Theta, U, nv, opts)
   % get matrix sizes
   ne = size(C,1);   % number of elements
   nc = size(U,2);   % number of constraints
   % specify default number of eigenvectors
   if (nargin < 4), nv = min(ne,16); end
   % specify default eigensolver options
   if (nargin < 5)
      opts.issym = 0;
      opts.isreal = 0;
      opts.disp = 0;
   end
   % compute confidence degree matrix
   C1 = sum(C,2);
   D_sqrt_inv = spdiags(1./(sqrt(C1)+eps),0,ne,ne);
   % compute generalized affinity matrix
   [x y conf] = find(C);
   inds = sub2ind([ne ne],x,y);
   ordering = full(Theta(inds));
   W = sparse(x,y,conf.*exp(i.*ordering),ne,ne);
   % transform eigenproblem into standard form
   P = D_sqrt_inv*W*D_sqrt_inv;
   A = D_sqrt_inv*U;
   B = inv(A'*A);
   % compute eigenvectors
   [evecs evals] = eigs(@(x) barqpqz(x,P,A,B), ne, nv, 'lm', opts);
   % transform back to original problem
   evecs = repmat(full(diag(D_sqrt_inv)),[1 nv]).*evecs;
   evals = diag(1-evals);
end

% Eigensolver inner sparse matrix-vector multiplication.
%
% z = barqpqz(x,P,A,B)
%
% Input:
%    x = ne x 1  complex vector
%    P = ne x ne complex sparse matrix
%    A = ne x nc complex sparse matrix
%    B = nc x nc complex sparse matrix
%
% Output:
%    z = (I - A * B * A') * P * (I - A * B * A') * x    
function z = barqpqz(x,P,A,B)
   z = x - A * (B * (A' * x));
   x = P * z;
   z = x - A * (B * (A' * x));
end
