% Compute poselet part-part affinities.
%
% [i j c theta] = affinity_parts(poselet_hits, model, tau, upsilon)
%
% Input:
%    poselet_hits - poselets detected in the image
%    model        - learned models for all poselets
%    tau          - affinity distance scale parameter
%    upsilon      - affinity score scale parameter
%
% Output:
%    (i,j)        - indices into sparse matrix of part-part relationships
%    c            - confidence on pairwise relationships
%    theta        - relative pairwise ordering relationships
%
% The returned indices range over the part nodes, starting at 1 for the
% first part.
function [i j c theta] = affinity_parts(poselet_hits, model, tau, upsilon)
   % get number of poselets
   n_poselets = poselet_hits.size;
   % initialize constants for poselet software
   detection_config;
   % transform poselets into object pose hypotheses
   hyps = instantiate_hypotheses([model.hough_votes.hyp],poselet_hits);
   % compute pairwise distances between parts based on object pose
   d_mx = zeros([n_poselets n_poselets]);
   for pl1 = 1:n_poselets
      for pl2 = 1:n_poselets
         d_mx(pl1,pl2) = hyps(pl1).distance(hyps(pl2));
      end
   end
   % create index vectors
   i = repmat((1:n_poselets).', [1 n_poselets]);
   j = repmat((1:n_poselets),   [n_poselets 1]);
   i = [i(:)];
   j = [j(:)];
   % compute score matrix to regulate connection strength
   scores = double(poselet_hits.score);
   score_i = repmat(scores,   [1 n_poselets]);
   score_j = repmat(scores.', [n_poselets 1]);
   score_mx = min(score_i,score_j);
   % transform distances and scores into affinities
   c = exp(-[d_mx(:)]./tau).*exp(-(1-[score_mx(:)])./upsilon);
   % set relative angular separation to zero
   theta = zeros(size(c));
end
