% Compute constraints between pixels and the global surround (figure/ground 
% prior) node.
%
% [i j v] = constraints_fg_prior(mpb, reps, rho)
%
% Input:
%    mpb          - multiscale probability of boundary
%                   (stored in matrix the same size as the image)
%    reps         - indices of pixels chosen to participate in constraints
%    rho          - pixel affinity scale parameter
%
% Output:
%    (i,j)        - indices into sparse constraint matrix
%    v            - constraint matrix values at (i,j)
%
% Here only a single constraint, that the global surround node is the average 
% of all pixels, is returned.  Hence, the returned matrix is actually a sparse
% column vector, and the entries of j are all ones.  Indices i range over the 
% pixel nodes and global surround node, with the global surround tacked on 
% after the last pixel.
function [i j v] = constraints_fg_prior(mpb, reps, rho)
   % get image size
   [sx sy] = size(mpb);
   n_pixels = sx.*sy;
   % grab affinities to global surround
   aff_fg = exp(-mpb(reps)./rho);
   % construct constraints
   i = [(n_pixels + 1) reps].';
   j = ones(size(i));
   v = [1 -aff_fg./(sum(aff_fg) + eps)].';
end
