% Compute constraints between pixels and poselet part and surround nodes.
%
% [i j v] = ...
%    constraints_pixels_parts(mpb, reps, rho, poselet_hits, fg_masks, sigma)
%
% Input:
%    mpb          - multiscale probability of boundary
%                   (stored in matrix the same size as the image)
%    reps         - indices of pixels chosen to participate in constraints
%    rho          - pixel affinity scale parameter
%    poselet_hits - poselets detected in the image
%    fg_masks     - learned figure/ground masks for the poselets
%    sigma        - pixel-part mask affinity scale parameter
%
% Output:   
%    (i,j)        - indices into sparse constraint matrix
%    v            - constraint matrix values at (i,j)
%
% The returned row indices range over pixel, part, and surround nodes, starting
% at 1 for the first pixel node, with part nodes following the last pixel node 
% and surround nodes following the last part.  The returned column indices 
% range over the active constraints.
function [i j v] = ...
   constraints_pixels_parts(mpb, reps, rho, poselet_hits, fg_masks, sigma)
   % get image size
   [sx sy] = size(mpb);
   n_pixels = sx.*sy;
   % get number of poselets
   n_poselets = poselet_hits.size;
   % construct pixel -> representative mapping
   n_reps = numel(reps);
   pix_rep = zeros([sx sy]);
   pix_rep(reps) = (1:n_reps);
   % compute pixel-poselet connections (only active for representative pixels)
   px_ind_arr = cell([n_poselets 1]);  % indices of pixels covered by poselet
   px_fg_arr  = cell([n_poselets 1]);  % figure/ground mask value at each pixel
   px_pb_arr  = cell([n_poselets 1]);  % pb value at each pixel
   for n = 1:n_poselets
      % get mask
      bounds = round(double(poselet_hits.bounds(n,:)));
      id = double(poselet_hits.poselet_id(n));
      fg_mask = fg_masks{id};
      score = double(poselet_hits.score(n));
      % resize mask
      ys = bounds(1); ylen = bounds(3); ye = ys+ylen-1;
      xs = bounds(2); xlen = bounds(4); xe = xs+xlen-1;
      fg_mask = imresize(fg_mask,[xlen ylen],'bilinear');
      % trim mask to be within image
      ytrim_s = max(-ys+1,0); ytrim_e = max(ye-sy,0);
      xtrim_s = max(-xs+1,0); xtrim_e = max(xe-sx,0);
      fg_mask = fg_mask((1+xtrim_s):(end-xtrim_e),(1+ytrim_s):(end-ytrim_e));
      ys = max(ys,1); ye = min(ye,sy);
      xs = max(xs,1); xe = min(xe,sx);
      % extract representatives covered by mask
      mask_reps = pix_rep(xs:xe,ys:ye);
      mask_reps = mask_reps(:);
      inds = find(mask_reps);
      % store pixel indices
      px_ind_arr{n} = reps(mask_reps(inds));
      % store figure/ground mask values
      px_fg_arr{n} = (fg_mask(inds)).';
      % store pb values
      px_pb_arr{n} = mpb(px_ind_arr{n});
   end
   % construct constraints
   nc = 0;
   u_i = cell([2.*n_poselets 1]);
   u_j = cell([2.*n_poselets 1]);
   u_v = cell([2.*n_poselets 1]);
   for n = 1:n_poselets
      % get indices of poselet part and surround nodes
      i_part     = n_pixels + n;
      i_surround = n_pixels + n_poselets + n;
      % get pixel indices, figure/ground, and pb values
      px_ind = px_ind_arr{n};
      px_fg  = px_fg_arr{n};
      px_pb  = px_pb_arr{n};
      % remap fg to be in [-1,1] range
      px_fg = 2.*(px_fg - 0.5);
      % compute pixel to part/surround connection strengths
      px_affinity = exp((abs(px_fg)-1)./sigma).*exp(-px_pb./rho);
      % switch pixel connections to either part or surround node
      in_part     = find(px_fg > 0);
      in_surround = find(px_fg < 0);
      % create pixel-part constraint
      if (~isempty(in_part))
         % grab affinities to part
         aff_part = px_affinity(in_part);
         % add constraint
         nc = nc + 1;
         u_i{nc} = [i_part px_ind(in_part)];
         u_j{nc} = repmat(nc,size(u_i{nc}));
         u_v{nc} = [1 -aff_part./(sum(aff_part)+eps)];
      end
      % create pixel-surround constraint
      if (~isempty(in_surround))
         % graph affinities to surround
         aff_surround = px_affinity(in_surround);
         % add constraint
         nc = nc + 1;
         u_i{nc} = [i_surround px_ind(in_surround)];
         u_j{nc} = repmat(nc,size(u_i{nc}));
         u_v{nc} = [1 -aff_surround./(sum(aff_surround)+eps)];
      end
   end
   % form sparse matrix representation of constraints
   i = ([u_i{:}]).';
   j = ([u_j{:}]).';
   v = ([u_v{:}]).';
end
