% Extract object detections from eigenvector representation of the poselets.
%
% dets = evecs_decode_dets(scene, poselet_hits, params)
%
% Input:
%    scene.          - partially complete scene information containing:
%       ae              - angular embedding eigenvectors
%       fg              - global figure/ground ordering
%       seg             - image segmentation
%    poselet_hits    - poselets detected in the image
%    params          - parameters for grouping and detection (see: seg_obj)
%
% Output:
%    dets.           - object detections
%       poselet_hits    - poselets detected in image (copy of input)
%       obj_bounds      - predicted object bounds
%       obj_torsos      - predicted object torsos
%       px_obj_ids      - mapping of pixels -> object ids (zero for none)
%       obj_mask_inds   - cell array containing indices of object pixels
%       obj_mask_vals   - figure/ground ordering on object pixels
function dets = evecs_decode_dets(scene, poselet_hits, params)
   % get image size
   sx = size(scene.fg.fg_pixels,1);
   sy = size(scene.fg.fg_pixels,2);
   % get number of pixels, poselets
   n_pixels   = sx.*sy;
   n_poselets = poselet_hits.size;
   % normalize eigenvectors
   evecs = evecs_normalize(scene.ae.evecs);
   % compute distance between all pairs of poselets in embedding space
   d_mx = zeros([n_poselets n_poselets]);
   for v = 2:size(evecs,2);
      z = evecs((1:n_poselets)+n_pixels,v);
      w = scene.ae.weights(v);
      z1 = repmat(z,   [1 n_poselets]);
      z2 = repmat(z.', [n_poselets 1]);
      d_mx = d_mx + w.*abs(z1 - z2);
   end
   % set infinite distance between incompatible poselets
   % (resulting clusters can contain at most one of each poselet type)
   id1 = repmat(double(poselet_hits.poselet_id),   [1 n_poselets]);
   id2 = repmat(double(poselet_hits.poselet_id).', [n_poselets 1]);
   d_mx(find(id1 == id2)) = inf;
   d_mx(1:(n_poselets+1):end) = 0;
   % extract elements of distance matrix below diagonal, place in vector
   d_vect = squareform(d_mx);
   % hierarchical agglomerative clustering
   T = linkage(d_vect,'complete');
   src1     = T(:,1);
   src2     = T(:,2);
   threshes = T(:,3);
   % pick highest level of hierarchy below merge threshold
   lvl = max(find(threshes < params.det_merge_th));
   % initialize final cluster assignments
   cluster_ids = (1:(2.*n_poselets - 1)).';
   % back propagate merges up to the current level
   if (~isempty(lvl))
      for l = lvl:-1:1
         c_ind = l + n_poselets;
         s1 = src1(l);
         s2 = src2(l);
         cluster_ids(s1) = cluster_ids(c_ind);
         cluster_ids(s2) = cluster_ids(c_ind);
      end
   end
   % remap unique cluster ids for the poselets
   cluster_ids = cluster_ids(1:n_poselets);
   [ids i j] = unique(cluster_ids);
   ids = (1:numel(ids)).';
   cluster_ids = ids(j);
   % instantiate poselet hypotheses
   detection_config;
   hyps = instantiate_hypotheses([params.model.hough_votes.hyp],poselet_hits);
   % merge clusters with significant overlap in the image domain
   [torso_predictions bounds_predictions] = ...
      cluster2bounds(poselet_hits, hyps, cluster_ids, params.model, [sy sx]);
   % store poselet detections
   dets.poselet_hits = poselet_hits;
   % store object detections
   dets.obj_bounds = bounds_predictions;
   dets.obj_torsos = torso_predictions;
   % decide upon active detections
   det_members = dets.obj_bounds.src_idx;
   det_active  = (dets.obj_bounds.score > params.det_score_th);
   % assign pixels to object detections
   [px_ids px_dist] = project_objects( ...
      evecs, scene.ae.weights, ...
      det_members, det_active, ...
      poselet_hits, params.fg_masks, [sx sy] ...
   );
   % store pixel to object id mapping
   dets.px_obj_ids = px_ids;
   % extract figure/ground mask for each object detection
   n_obj = dets.obj_bounds.size;
   dets.obj_mask_inds = cell([n_obj 1]);
   dets.obj_mask_vals = cell([n_obj 1]);
   [obj_ids obj_pixels] = sort([px_ids(:)]);
   [junk o_start] = unique(obj_ids,'first');
   [junk o_end]   = unique(obj_ids,'last');
   for o = 1:numel(o_start)
      % get current object id
      o_id = obj_ids(o_start(o));
      if (o_id > 0)
         % get pixels assigned to object
         inds = obj_pixels(o_start(o):o_end(o));
         % store figure/ground mask for those pixels
         dets.obj_mask_inds{o_id} = inds;
         dets.obj_mask_vals{o_id} = scene.fg.fg_pixels(inds);
      end
   end
end
