% Extract figure/ground ordering from the first eigenvector.
%
% fg = evecs_decode_fg(evecs, ang_norm_factor, [sx sy])
%
% Input:
%    evecs           - eigenvectors sorted in ascending order by eigenvalue
%    ang_norm_factor - normalization factor for angular span of embedding
%    [sx sy]         - size of image (in pixels)
%
% Output:
%    fg.             - global figure/ground ordering
%       fg_pixels       - figure/ground ordering on pixels
%       fg_parts        - figure/ground ordering on parts
%       fg_surround     - figure/ground ordering on surround nodes
%       fg_f            - figure/ground rank of global surround node
function fg = evecs_decode_fg(evecs, ang_norm_factor, im_size)
   % extract image size arguments
   sx = im_size(1);
   sy = im_size(2);
   % get number of pixels, poselets
   n_nodes    = size(evecs,1);
   n_pixels   = sx.*sy;
   n_poselets = (n_nodes - n_pixels - 1)./2;
   % extract the angle of the first eigenvector as global ordering
   ordering = angle(evecs(:,1));
   % rotate so the minimum angle coincides with the x-axis
   % (all angles will be positive as a result)
   ordering = ordering - min(ordering);
   % angular span of ordering is guaranteed to be < pi radians:
   % if this is not the case, then we need to invert the ordering
   % (due to the embedding crossing the negative real axis)
   if (max(ordering) > pi)
      ordering = 2.*pi - ordering;
      ordering = ordering - min(ordering);
   end
   % rescale by normalization factor 
   ordering = ordering.*ang_norm_factor;
   % extract pixel, part, surround, and global surround components
   fg.fg_pixels   = reshape(ordering(1:n_pixels),[sx sy]);
   fg.fg_parts    = ordering((1:n_poselets)+n_pixels);
   fg.fg_surround = ordering((1:n_poselets)+n_pixels+n_poselets);
   fg.fg_f        = ordering(n_nodes);
end
