% Extract segmentation from sorted eigenvectors.
%
% seg = evecs_decode_seg(evecs, weights, [sx sy])
%
% Input:
%    evecs           - eigenvectors sorted in ascending order by eigenvalue
%    weights         - weight on each eigenvector
%    [sx sy]         - size of image (in pixels)
%
% Output:
%    seg.            - image segmentation
%       spb_ori         - spectral pb contours (oriented signal)
%       spb             - spectral pb contours (max over orientations)
%       ucm             - hierarchical segmentation (ultrametric contour map)
function seg = evecs_decode_seg(evecs, weights, im_size)
   % get number of eigenvectors, number of pixels
   nv = size(evecs,2);
   sx = im_size(1);
   sy = im_size(2);
   n_pixels = sx.*sy;
   % grab portion of eigenvectors encoding pixel segmentation
   evecs   = evecs(1:n_pixels,2:end);
   weights = weights(2:end);
   % normalize eigenvectors
   evecs = evecs_normalize(evecs);
   % normalize weights
   weights = weights./(sum(weights) + eps);
   % compute spectral pb signal
   evecs = reshape(evecs, [sx sy 1 (nv-1)]);
   [spb spb_ori] = evecs_to_edges(evecs, weights);
   seg.spb_ori = spb_ori;
   seg.spb     = spb;
   % construct hierarchical segmentation
   seg.ucm = contours2ucm(spb_ori./max(spb_ori(:)), 'doubleSize');
end

% Extract edges from eigenvectors.
function [pb pbos] = evecs_to_edges(evecs, weights)
   % get size
   [sx sy nscale nvec] = size(evecs);
   % gradient parameters
   hil = 0;
   deriv = 1;
   support = 3;
   sigma = 1;
   norient = 8;
   dtheta = pi/norient;
   ch_per = [4 3 2 1 8 7 6 5];
   % compute pixel gradients
   pbos = zeros([sx sy norient nscale]);
   for scale = 1:nscale
      for v = 1:nvec
         if (weights(v) > 0),
            vec = evecs(:,:,scale,v).*weights(v);
            for o = 1:norient,
               theta = dtheta*o;
               f = oeFilter(sigma, support, theta, deriv, hil);
               pbos(:,:,ch_per(o),scale) = ...
                  pbos(:,:,ch_per(o),scale) + abs(applyFilter(f, vec));
            end
         end
      end
   end
   % sum over scales
   pb = sum(pbos,4);
   % take maximum over orientations
   pb = max(pb,[],3);
   % normalize
   pb = pb./max(pb(:));
end
