% Project object detections onto the pixel grid.
%
% Assign each pixel the id of the object containing the closest poselet 
% covering it.  The distance from a pixel to a poselet is computed using 
% their respective representations in the embedding space.
%
% An additional argument optionally specifies that only a subset of the 
% object detections should be considered active.
%
% [px_ids px_dist] = project_objects( ...
%    evecs, weights, det_members, det_active, poselet_hits, fg_masks, [sx sy])
%
% Input:
%    evecs           - normalized eigenvectors in ascending order by eigenvalue
%    weights         - weight on each eigenvector
%    det_members     - cell array containing list of poselets in each object
%    det_active      - is object detection active?
%    poselet_hits    - poselets detected in the image
%    fg_masks        - learned figure/ground masks for the poselets
%    [sx sy]         - size of image (in pixels)
%
% Output:
%    px_ids          - mapping of pixels to object ids
%                      (zero for unassigned pixels)
%    px_dist         - distance between pixel and object assigned to it
%                      (inf for unassigned pixels)
function [px_ids px_dist] = project_objects( ...
   evecs, weights, det_members, det_active, poselet_hits, fg_masks, im_size)
   % get number of objects, poselets
   n_obj      = numel(det_members);
   n_poselets = poselet_hits.size;
   % compute: set of active poselets, mapping of poselets -> objects
   poselet_active = zeros([n_poselets 1]);
   obj_ids        = zeros([n_poselets 1]);
   for n = 1:n_obj
      % check if object is active
      if (det_active(n))
         % get poselets belonging to object
         poselet_ids = det_members{n};
         % mark poselets as active
         poselet_active(poselet_ids) = 1;
         % store poselet -> object mapping
         obj_ids(poselet_ids) = n;
      end
   end
   % project poselets onto pixels
   [px_ids px_dist] = project_poselets( ...
      evecs, weights, poselet_hits, poselet_active, fg_masks, im_size);
   % translate poselet ownership to object ownership
   inds = find(px_ids > 0);
   px_ids(inds) = obj_ids(px_ids(inds));
end
