% Project poselet ids onto the pixel grid.
%
% Assign each pixel the id of the closest poselet covering it (if any).  The 
% distance from a pixel to a poselet is computed using their respective 
% representations in the embedding space.
%
% An additional argument optionally specifies that only a subset of the 
% poselets should be examined.
%
% [px_ids px_dist] = project_poselets( ...
%    evecs, weights, poselet_hits, poselet_active, fg_masks, [sx sy])
%
% Input:
%    evecs           - normalized eigenvectors in ascending order by eigenvalue
%    weights         - weight on each eigenvector
%    poselet_hits    - poselets detected in the image
%    poselet_active  - is poselet active?
%    fg_masks        - learned figure/ground masks for the poselets
%    [sx sy]         - size of image (in pixels)
%
% Output:
%    px_ids          - mapping of pixels to poselet ids
%                      (zero for unassigned pixels)
%    px_dist         - distance between pixel and poselet assigned to it
%                      (inf for unassigned pixels)
function [px_ids px_dist] = project_poselets( ...
   evecs, weights, poselet_hits, poselet_active, fg_masks, im_size)
   % get number of eigenvectors
   nv = size(evecs,2);
   % get number of pixels
   sx = im_size(1);
   sy = im_size(2);
   n_pixels = sx.*sy;
   % get number of poselets
   n_poselets = poselet_hits.size;
   % project poselets onto pixels
   px_ids  = zeros([sx sy]);
   px_dist = repmat(inf,[sx sy]);
   for n = 1:n_poselets
      % check if poselet is active
      if (poselet_active(n))
         % get mask
         bounds = round(double(poselet_hits.bounds(n,:)));
         id = double(poselet_hits.poselet_id(n));
         fg_mask = fg_masks{id};
         % resize mask
         ys = bounds(1); ylen = bounds(3); ye = ys+ylen-1;
         xs = bounds(2); xlen = bounds(4); xe = xs+xlen-1;
         fg_mask = imresize(fg_mask,[xlen ylen],'bilinear');
         % trim mask to be within image
         ytrim_s = max(-ys+1,0); ytrim_e = max(ye-sy,0);
         xtrim_s = max(-xs+1,0); xtrim_e = max(xe-sx,0);
         fg_mask = fg_mask((1+xtrim_s):(end-xtrim_e),(1+ytrim_s):(end-ytrim_e));
         ys = max(ys,1); ye = min(ye,sy);
         xs = max(xs,1); xe = min(xe,sx);
         % get indices into image
         x_inds = repmat((xs:xe).', [1 numel(ys:ye)]);
         y_inds = repmat((ys:ye),   [numel(xs:xe) 1]);
         x_inds = [x_inds(:)];
         y_inds = [y_inds(:)];
         inds = sub2ind([sx sy], x_inds, y_inds);
         % compute distance between pixels and poselet in embedding space
         np = numel(inds);
         z_poselet = repmat(evecs(n_pixels + n,2:end),[np 1]);
         z_pixels  = evecs(inds,2:end);
         d = repmat(weights(2:end).',[np 1]).*abs(z_pixels - z_poselet);
         d = sum(d,2);
         % lookup current distances
         d_curr = px_dist(inds);
         d_curr = [d_curr(:)];
         % determine indices to update
         j = find(d < d_curr);
         inds = inds(j);
         % update pixel assignments
         px_ids(inds)  = n;
         px_dist(inds) = d(j);
      end
   end
end
