% Figure/ground, segmentation, and object detection.
% 
% scene = seg_obj(pb, poselet_hits, [params])
%
% params = seg_obj('params') % returns default parameter settings
%
% Input:
%    pb              - multiscale pb components (bg, cg, tg)
%    poselet_hits    - poselets detected in the image
%    params.         - parameters (optional, defaults used if not specified)
%       model           - learned poselet model
%       fg_masks        - learned poselet figure/ground masks
%       px_samp         - fraction of pixels to sample for constraints
%       w_alpha         - relative importance of part clustering subproblem
%       w_beta          - relative importance of part-surround subproblem
%       w_gamma         - relative importance of figure/ground prior
%       aff_rho         - scale parameter for pixel-pixel affinities
%       aff_sigma       - scale parameter for pixel-part mask membership
%       aff_tau         - scale parameter for part-part distance affinity
%       aff_upsilon     - scale parameter for part-part score affinity
%       det_merge_th    - threshold on agglomerative merging of parts
%       det_score_th    - threshold on scores of final object detections
%       n_evec          - # of angular embedding eigenvectors to use
%
% Output:
%    scene.          - interpretation of multiple aspects of the scene
%       ae.             - angular embedding eigenvectors
%          evecs           - complex eigenvectors from angular embedding
%          evals           - eigenvalues from angular embedding
%          weights         - relative weighting of eigenvectors
%          ang_norm_factor - angular normalization factor
%       fg.             - global figure/ground ordering
%          fg_pixels       - figure/ground ordering on pixels
%          fg_parts        - figure/ground ordering on parts
%          fg_surround     - figure/ground ordering on surround nodes
%          fg_f            - figure/ground rank of global surround node
%       seg.            - image segmentation
%          spb_ori         - spectral pb contours (oriented signal)
%          spb             - spectral pb contours (max over orientations)
%          ucm             - hierarchical segmentation (ultrametric contour map)
%       dets.           - object detections
%          poselet_hits    - poselets detected in image (copy of input)
%          obj_bounds      - predicted object bounds
%          obj_torsos      - predicted object torsos
%          px_obj_ids      - mapping of pixels -> object ids (zero for none)
%          obj_mask_inds   - cell array containing indices of object pixels
%          obj_mask_vals   - figure/ground ordering on object pixels
function scene = seg_obj(pb, poselet_hits, params)
   % default parameters - poselet model
   poselets_model = load_poselets('poselets_model_sep10.mat');
   params_def.model    = poselets_model.model;
   params_def.fg_masks = poselets_model.fg_masks;
   % default parameters - pixel sampling
   params_def.px_samp      = 0.05;  % frac of pixels to sample for constraints
   % default parameters - subproblem importance 
   params_def.w_alpha      = 0.002; % part-part
   params_def.w_beta       = 0.5;   % part-surround
   params_def.w_gamma      = 0.1;   % part-global surround (figure/ground prior)
   % default parameters - affinity scaling
   params_def.aff_rho      = 0.1;   % pixel-pixel affinity
   params_def.aff_sigma    = 0.1;   % pixel-part mask membership affinity
   params_def.aff_tau      = 100;   % part-part distance affinity
   params_def.aff_upsilon  = 0.1;   % part-part score affinity
   % default parameters - detection thresholds
   params_def.det_merge_th = 500;   % part agglomerative merging threshold
   params_def.det_score_th = 12;    % object detection score threshold
   % default parameters - number of eigenvectors
   params_def.n_evec = 16;
   % check if returning default parameters requested
   if ((nargin == 1) && (ischar(pb)))
      % return default parameters
      scene = params_def;
      return;
   end
   % check arguments
   if ((nargin < 3) || isempty(params))
      params = params_def;
   end
   % set unspecified parameters to defaults
   params = set_defaults(params, params_def);
   % initialize output
   scene = scene_init;
   % construct multiscale pb from components
   mpb_oriented = mPb_all_from_cues( ...
      pb.bg1,  pb.bg2,  pb.bg3, ...
      pb.cga1, pb.cga2, pb.cga3, ...
      pb.cgb1, pb.cgb2, pb.cgb3, ...
      pb.tg1,  pb.tg2,  pb.tg3, 1.0 ...
   );
   mpb = max(mpb_oriented,[],3);
   % get number of pixels, poselets, and nodes
   [sx sy] = size(mpb);
   n_pixels   = sx.*sy;
   n_poselets = poselet_hits.size;
   n_nodes    = n_pixels + 2.*n_poselets + 1;
   % compute affinities
   [i_p j_p c_p theta_p] = affinity_pixels(mpb, params.aff_rho);
   [i_q j_q c_q theta_q] = affinity_parts( ...
                              poselet_hits, params.model, ...
                              params.aff_tau, params.aff_upsilon);
   [i_s j_s c_s theta_s] = affinity_parts_surround( ...
                              poselet_hits, params.aff_upsilon);
   [i_f j_f c_f theta_f] = affinity_fg_prior( ...
                              poselet_hits, params.aff_upsilon);
   % sample representative pixels to participate in constraints
   n_reps = ceil((n_pixels).*(params.px_samp));
   reps = randperm(n_pixels);
   reps = reps(1:n_reps);
   % compute constraints
   [ui_q uj_q uv_q] = constraints_pixels_parts( ...
                         mpb, reps, params.aff_rho, ...
                         poselet_hits, params.fg_masks, params.aff_sigma);
   [ui_f uj_f uv_f] = constraints_fg_prior(mpb, reps, params.aff_rho);
   % get total number of constraints
   n_constraints = max(uj_q(:)) + 1;
   % adjust affinity matrix indices to reference nodes in the combined graph
   i_q = i_q + n_pixels;  j_q = j_q + n_pixels;
   i_s = i_s + n_pixels;  j_s = j_s + n_pixels;
   i_f = i_f + n_pixels;  j_f = j_f + n_pixels;
   % adjust constraint matrix indices to reference nodes in the combined graph
   ui_f(find(ui_f == (n_pixels+1))) = n_nodes;
   uj_f = repmat(n_constraints, size(uj_f));
   % assemble affinity matrix indices, confidence and ordering values
   i = cat(1, i_p, i_q, i_s, j_s, i_f, j_f);
   j = cat(1, j_p, j_q, j_s, i_s, j_f, i_f);
   c_vals = cat(1, ...
      c_p, ...
      (params.w_alpha).*c_q, ...
      (params.w_beta).*c_s,  (params.w_beta).*c_s, ...
      (params.w_gamma).*c_f, (params.w_gamma).*c_f ...
   );
   theta_vals = cat(1, ...
      theta_p, ...
      theta_q, ...
      theta_s, -theta_s, ...
      theta_f, -theta_f ...
   );
   % normalize angular span
   scene.ae.ang_norm_factor = sum(abs(theta_vals));
   theta_vals = theta_vals./(scene.ae.ang_norm_factor + eps);
   % assemble confidence and ordering matrices
   C     = sparse(i, j, c_vals,     n_nodes, n_nodes);
   Theta = sparse(i, j, theta_vals, n_nodes, n_nodes);
   % assemble constraint matrix
   ui = cat(1, ui_q, ui_f);
   uj = cat(1, uj_q, uj_f);
   uv = cat(1, uv_q, uv_f);
   U = sparse(ui, uj, uv, n_nodes, n_constraints);
   % solve constrained angular embedding problem
   opts.issym = 0;
   opts.isreal = 0;
   opts.disp = 1;
   tic;
   [evecs evals] = ae_constrained(C, Theta, U, params.n_evec, opts);
   toc;
   % sort eigenvectors by weight
   [evecs evals weights] = evecs_sort(evecs, evals);
   % store eigenvectors
   scene.ae.evecs   = evecs;
   scene.ae.evals   = evals;
   scene.ae.weights = weights;
   % decode eigenvectors
   scene.fg   = evecs_decode_fg(evecs, scene.ae.ang_norm_factor, [sx sy]);
   scene.seg  = evecs_decode_seg(evecs, weights, [sx sy]);
   scene.dets = evecs_decode_dets(scene, poselet_hits, params);
end

% Initialize output scene data structure.
function scene = scene_init()
   % angular embedding results
   scene.ae.evecs           = [];
   scene.ae.evals           = [];
   scene.ae.weights         = [];
   scene.ae.ang_norm_factor = [];
   % figure/ground
   scene.fg.fg_pixels       = [];
   scene.fg.fg_parts        = [];
   scene.fg.fg_surround     = [];
   scene.fg.fg_f            = [];
   % segmentation
   scene.seg.spb_ori        = [];
   scene.seg.spb            = [];
   scene.seg.ucm            = [];
   % object detections
   scene.dets.poselet_hits  = [];
   scene.dets.obj_bounds    = [];
   scene.dets.obj_torsos    = [];
   scene.dets.px_obj_ids    = [];
   scene.dets.obj_mask_inds = [];
   scene.dets.obj_mask_vals = [];
end
