% Generate visualization of scene globalization results.
%
% vis = vis_generate(im, scene, [options])
%
% options = vis_generate('options') % returns default options
%
% Input:
%    im              - original image
%    scene           - interpretation of scene (generated by seg_obj)
%    options.        - options controlling visualization output
%       model           - learned poselet model
%       fg_masks        - learned poselet masks
%       bbox_width      - width of bounding boxes (in pixels)
%       keypoint_width  - width of keypoints (in pixels)
%       keypoint_ids    - ids of part keypoints to display
%
% Output:
%    vis.            - visualization data
%       im              - original image
%       fg              - figure/ground ordering on pixels
%       fg_im           - figural portion of the image
%       ucm             - hierarchical segmentation (ultrametric contour map)
%       det_fg          - figure/ground from top-down object detections
%       bbox            - bounding boxes for all object detections
%       bbox_color      - bounding boxes overlayed on color image
%       obj_bbox        - object detection bounding boxes
%       obj_clus        - object detection part clusters
%       obj_fg          - object instance specific figure/ground
%       obj_im          - objects extracted from the image
%       obj_scores      - object detection scores
function vis = visualize_scene(im, scene, options)
   % default options - poselet model
   poselets_model = load_poselets('poselets_model_sep10.mat');
   options_def.model    = poselets_model.model;
   options_def.fg_masks = poselets_model.fg_masks;
   % default options - display
   options_def.bbox_width     = 10;    % bound box width
   options_def.keypoint_width = 5;     % keypoint width
   options_def.keypoint_ids   = 1:20;  % keypoints to display
   % check if returning default options requested
   if ((nargin == 1) && (ischar(im)))
      % return default parameters
      vis = options_def;
      return;
   end
   % check arguments
   if ((nargin < 3) || isempty(options))
      options = options_def;
   end
   % set unspecified options to defaults
   options = set_defaults(options, options_def);
   % store image
   vis.im = im;
   % get grayscale version of image
   if (size(im,3)>1)
      img = rgb2gray(im);
   else
      img = im;
      im = repmat(im,[1 1 3]);
   end
   % store normalized figure/ground ordering on pixels
   vis.fg = ...
      (scene.fg.fg_pixels - min(scene.fg.fg_pixels(:))) ./ ...
      (max(scene.fg.fg_pixels(:)) - min(scene.fg.fg_pixels(:)) + eps);
   % create display of figural portion of image
   vis.fg_im = ...
      repmat(vis.fg,[1 1 3]).*im + ...
      repmat(1-vis.fg,[1 1 3]).*0.5;
   % store ucm, in image-size format
   vis.ucm = 1 - (scene.seg.ucm(3:2:end,3:2:end));
   % convert poselets to hypotheses
   detection_config;
   hyps = instantiate_hypotheses( ...
      [options.model.hough_votes.hyp], scene.dets.poselet_hits);
   img = repmat(img,[1 1 3]);
   % get ids and scores of detected objects
   obj_ids = unique(scene.dets.px_obj_ids);
   obj_ids = obj_ids(find(obj_ids > 0));
   obj_scores = double(scene.dets.obj_bounds.score);
   obj_scores = obj_scores(obj_ids);
   % sort object ids by score
   [obj_scores sc_inds] = sort(obj_scores,'descend');
   obj_ids = obj_ids(sc_inds);
   n_obj = numel(obj_ids);
   % render top-down (poselet average) view of object detections
   vis.det_fg = obj_mask_dets( ...
      scene.dets.obj_bounds.select(obj_ids), ...
      scene.dets.poselet_hits, ...
      options.fg_masks, ...
      size(scene.fg.fg_pixels) ...
   );
   vis.det_fg = ...
      (vis.det_fg - min(vis.det_fg(:))) ./ ...
      (max(vis.det_fg(:)) - min(vis.det_fg(:)) + eps);
   % initialize object detection bounding boxes
   [sx sy] = size(vis.fg);
   vis.bbox       = [];
   vis.bbox_color = [];
   vis.obj_bbox = cell([n_obj 1]);
   % render object detection bounding boxes
   bb_all = zeros([sx sy]);
   for n = 1:n_obj
      % get bounds
      o = obj_ids(n);
      bounds = round(double(scene.dets.obj_bounds.bounds(o,:)));
      ys = bounds(1); ylen = bounds(3); ye = ys+ylen-1;
      xs = bounds(2); xlen = bounds(4); xe = xs+xlen-1;
      % move bounds to inside width of image border
      w = options.bbox_width - 1;
      xs = min(xs,sx-w); xe = max(xe, w+1);
      ys = min(ys,sy-w); ye = max(ye, w+1);
      % render bounding box
      bb = zeros([sx sy]);
      bb(xs:(xs+w),ys:ye) = 1;
      bb((xe-w):xe,ys:ye) = 1;
      bb(xs:xe,ys:(ys+w)) = 1;
      bb(xs:xe,(ye-w):ye) = 1;
      % add to all bounding boxes
      bb_inds = find(bb);
      bb_all(bb_inds) = 1;
      % overlay bounding box on grayscale image
      bb_r = img(:,:,1); bb_g = img(:,:,2); bb_b = img(:,:,3);
      bb_r(bb_inds) = 1; bb_g(bb_inds) = 0; bb_b(bb_inds) = 0;
      % store overlay
      vis.obj_bbox{n} = cat(3, bb_r, bb_g, bb_b);
   end
   % get pixels covered by bounding boxes
   bb_inds = find(bb_all);
   % overlay all bounding boxes on grayscale image
   bb_r = img(:,:,1); bb_g = img(:,:,2); bb_b = img(:,:,3);
   bb_r(bb_inds) = 1; bb_g(bb_inds) = 0; bb_b(bb_inds) = 0;
   vis.bbox = cat(3, bb_r, bb_g, bb_b);
   % overlay all bounding boxes on color image
   bb_r = im(:,:,1); bb_g = im(:,:,2); bb_b = im(:,:,3);
   bb_r(bb_inds) = 1; bb_g(bb_inds) = 0; bb_b(bb_inds) = 0;
   vis.bbox_color = cat(3, bb_r, bb_g, bb_b);
   % render object detection part clusters
   vis.obj_clus = cell([n_obj 1]);
   for n = 1:n_obj
      % get object id
      o = obj_ids(n);
      % render keypoint predictions
      poselet_ids = scene.dets.obj_bounds.src_idx{o};
      hs = hyps(poselet_ids);
      vis.obj_clus{n} = render_keypoints( ...
         img, hs, options.keypoint_ids, options.keypoint_width);
   end
   % render object instance masks
   vis.obj_fg = cell([n_obj 1]);
   vis.obj_im = cell([n_obj 1]);
   for n = 1:n_obj
      o = obj_ids(n);
      obj_fg_mask = (scene.dets.px_obj_ids==o).*(vis.fg);
      vis.obj_fg{n} = obj_fg_mask;
      vis.obj_im{n} = ...
         repmat(obj_fg_mask,[1 1 3]).*im + ...
         repmat(1-obj_fg_mask,[1 1 3]).*0.5;
   end
   % store object detection scores
   vis.obj_scores = obj_scores;
end

% Render hypothesis keypoints.
function frm = render_keypoints(img, hs, keypoint_ids, w)
   % colors for keypoints
   colors = jet(numel(keypoint_ids));
   % initialize keypoint overlay
   [sx sy sz] = size(img);
   ovly_r = zeros([sx sy]);
   ovly_g = zeros([sx sy]);
   ovly_b = zeros([sx sy]);
   % draw keypoints of each hypothesis
   for n = 1:numel(hs)
      % get hypothesis
      h = hs(n);
      % get colors of keypoints within image
      x = round(h.mu(keypoint_ids,2));
      y = round(h.mu(keypoint_ids,1));
      in_bounds = ((1 <= x) & (x <= sx) & (1 <= y) & (y <= sy));
      inds = find(in_bounds);
      x = x(inds);
      y = y(inds);
      c_r = colors(inds,1);
      c_g = colors(inds,2);
      c_b = colors(inds,3);
      % update overlay
      ii = sub2ind([sx sy],x,y);
      ovly_r(ii) = max(ovly_r(ii),c_r);
      ovly_g(ii) = max(ovly_g(ii),c_g);
      ovly_b(ii) = max(ovly_b(ii),c_b);
   end
   % convolve overlay
   f = ones([1 w]);
   ovly_r = conv2(conv2(ovly_r,f,'same'),f.','same');
   ovly_g = conv2(conv2(ovly_g,f,'same'),f.','same');
   ovly_b = conv2(conv2(ovly_b,f,'same'),f.','same');
   ovly = min(cat(3,ovly_r,ovly_g,ovly_b),1);
   % combine overlay with image
   o_mask = repmat((max(ovly,[],3) > 0),[1 1 3]);
   frm = o_mask.*ovly + (1-o_mask).*img;
end
