% Compute segment features for use in refinement.
%
% [sdata sfeat] = seg_features(scene, im, [im_gt], [params])
%
% params = seg_features('params') % returns default parameter settings
%
% Input:
%    scene           - interpretation of scene (created by seg_obj)
%    im              - original image
%    im_gt           - groundtruth segmentation mask
%    params.         - parameters for feature generation
%       params_scene    - parameters used by seg_obj
%       ucm_ths         - UCM thresholds at which to form segmentation
%       gt_class_id     - groundtruth class identity
%       gt_any_id       - groundtruth "don't care" identity
%
% Output:
%    sdata.          - segment data structure
%       ucm_arr         - UCM at each threshold
%       seg_arr         - segment graph (created by segment_ucm)
%       id_map_arr      - map of finest regions onto coarser levels
%    sfeat.          - features for each segment
%       obj_id          - object to which segment belongs (zero for none)
%       obj_score       - object detection score for segment
%       gt_overlap      - groundtruth overlap (if available, zero otherwise)
%       f.              - features at each level in ucm, mapped to base segment
%          seg_size        - segment size (in pixels)
%          frac_size       - segment size (as fraction of bounding box area)
%          fg_avg          - mean figural ranking
%          fg_norm_avg     - mean normalized (to image max) figural ranking
%          fg_diff_avg     - mean figure/ground difference from surroundings
%          fg_diff_max     - max figure/ground difference from surroundings
%          pl_avg          - mean poselet mask strength
%          pl_diff_avg     - mean poselet mask strength diff from surroundings
%          pl_diff_max     - max poselet mask strength diff from surroundings
%          cov_fg          - fraction of segment with > 0.50 normalized fg rank
%          cov_pl          - fraction of segment > poselet foreground cutoff
%          aff_obj_center  - affinity to object bounding box center
%          aff_obj_ur      - affinity to object bounding box upper right
%          aff_obj_ul      - affinity to object bounding box upper left
%          aff_obj_lr      - affinity to object bounding box lower right
%          aff_obj_ll      - affinity to object bounding box lower left
%          aff_obj_x       - affinity to object center x coordinate
%          aff_obj_y       - affinity to object center y coordinate
function [sdata sfeat] = seg_features(scene, im, im_gt, params)
   % default parameters
   params_def.params_scene = seg_obj('params');
   %params_def.ucm_ths      = [0.025 0.050 0.075 0.100];
   %params_def.ucm_ths      = [0.0250 0.0375 0.0500];
   params_def.ucm_ths      = [0.0250];
   params_def.gt_class_id  = 15;    % person class
   params_def.gt_any_id    = 255;   % border pixels which are "don't care"
   % check if returning default parameters requested
   if ((nargin == 1) && (ischar(scene)))
      % return default parameters
      sdata = params_def;
      return;
   end
   % get image size
   [sx sy sz] = size(im);
   % check arguments
   if ((nargin < 3) || isempty(im_gt))
      im_gt = zeros([sx sy]);
   end
   if ((nargin < 4) || isempty(params))
      params = params_def;
   end
   % set unspecified parameters to defaults
   params = set_defaults(params, params_def);
   % compute segment data - threshold and segment ucm at multiple levels
   n_ucm_th = numel(params.ucm_ths);
   sdata.ucm_arr    = cell([n_ucm_th 1]);
   sdata.seg_arr    = cell([n_ucm_th 1]);
   sdata.id_map_arr = cell([n_ucm_th 1]);
   for lvl = 1:n_ucm_th
      sdata.ucm_arr{lvl} = ...
         scene.seg.ucm .* (scene.seg.ucm > params.ucm_ths(lvl));
      sdata.seg_arr{lvl} = ...
         segment_ucm(sdata.ucm_arr{lvl});
   end
   % compute segment data - get representative pixels at finest level
   n_reg_base  = sdata.seg_arr{1}.n_regions;
   reg_base_px = zeros([n_reg_base 1]);
   for r = 1:n_reg_base
      reg_base_px(r) = sdata.seg_arr{1}.seg_members{r}(1);
   end
   % compute segment data - map finest level regions onto coarse level
   for lvl = 1:n_ucm_th
      sdata.id_map_arr{lvl} = sdata.seg_arr{lvl}.seg(reg_base_px);
   end
   % create mask for globalization results
   o_ids = ...
      find(scene.dets.obj_bounds.score > params.params_scene.det_score_th);
   pl_mask = obj_mask_dets( ...
      scene.dets.obj_bounds.select(o_ids), ...
      scene.dets.poselet_hits, ...
      params.params_scene.fg_masks, ...
      [sx sy] ...
   );
   % extract pixel figural ranking
   fg_mask      = scene.fg.fg_pixels;
   fg_mask      = fg_mask - min(fg_mask(:));
   fg_mask_norm = fg_mask./(max(fg_mask(:)) + eps);
   % initialize features for base level region only
   sfeat.obj_id            = zeros([n_reg_base 1]);
   sfeat.obj_score         = zeros([n_reg_base 1]);
   sfeat.gt_overlap        = zeros([n_reg_base 1]);
   % initialize features across all levels
   sfeat.f.seg_size        = zeros([n_reg_base n_ucm_th]);
   sfeat.f.frac_size       = zeros([n_reg_base n_ucm_th]);
   sfeat.f.fg_avg          = zeros([n_reg_base n_ucm_th]);
   sfeat.f.fg_norm_avg     = zeros([n_reg_base n_ucm_th]);
   sfeat.f.fg_diff_avg     = zeros([n_reg_base n_ucm_th]);
   sfeat.f.fg_diff_max     = zeros([n_reg_base n_ucm_th]);
   sfeat.f.pl_avg          = zeros([n_reg_base n_ucm_th]);
   sfeat.f.pl_diff_avg     = zeros([n_reg_base n_ucm_th]);
   sfeat.f.pl_diff_max     = zeros([n_reg_base n_ucm_th]);
   sfeat.f.cov_fg          = zeros([n_reg_base n_ucm_th]);
   sfeat.f.cov_pl          = zeros([n_reg_base n_ucm_th]);
   sfeat.f.aff_obj_center  = zeros([n_reg_base n_ucm_th]);
   sfeat.f.aff_obj_ur      = zeros([n_reg_base n_ucm_th]);
   sfeat.f.aff_obj_ul      = zeros([n_reg_base n_ucm_th]);
   sfeat.f.aff_obj_lr      = zeros([n_reg_base n_ucm_th]);
   sfeat.f.aff_obj_ll      = zeros([n_reg_base n_ucm_th]);
   sfeat.f.aff_obj_x      = zeros([n_reg_base n_ucm_th]);
   sfeat.f.aff_obj_y      = zeros([n_reg_base n_ucm_th]);
   % compute features specific to base ucm level
   for r = 1:n_reg_base
      % get pixel membership of current region
      px = sdata.seg_arr{1}.seg_members{r};
      % get object id for region and lookup object score
      sfeat.obj_id(r) = mode(scene.dets.px_obj_ids(px));
      if (sfeat.obj_id(r) > 0)
         sfeat.obj_score(r) = scene.dets.obj_bounds.score(sfeat.obj_id(r));
      end
      % compute groundtruth overlap
      sfeat.gt_overlap(r) = ...
         numel(find(im_gt(px) == params.gt_class_id))./(numel(px));
   end
   % compute features at each ucm level
   for lvl = 1:n_ucm_th
      % get number of regions in level
      n_reg = sdata.seg_arr{lvl}.n_regions;
      % initialize region features
      reg.seg_size        = zeros([n_reg 1]);
      reg.fg_avg          = zeros([n_reg 1]);
      reg.fg_norm_avg     = zeros([n_reg 1]);
      reg.fg_diff_avg     = zeros([n_reg 1]);
      reg.fg_diff_max     = zeros([n_reg 1]);
      reg.pl_avg          = zeros([n_reg 1]);
      reg.pl_diff_avg     = zeros([n_reg 1]);
      reg.pl_diff_max     = zeros([n_reg 1]);
      reg.cov_fg          = zeros([n_reg 1]);
      reg.cov_pl          = zeros([n_reg 1]);
      reg.center_x        = zeros([n_reg 1]);
      reg.center_y        = zeros([n_reg 1]);
      % compute region unary features
      for r = 1:n_reg
         % get pixel membership of current region
         px = sdata.seg_arr{lvl}.seg_members{r};
         % region size
         reg.seg_size(r) = numel(px);
         % region fg mask averages
         reg.fg_avg(r)      = mean(fg_mask(px));
         reg.fg_norm_avg(r) = mean(fg_mask_norm(px));
         reg.pl_avg(r)      = mean(pl_mask(px));
         % region fg coverage fraction
         reg.cov_fg(r) = numel(find(fg_mask_norm(px) > 0.50))./numel(px);
         reg.cov_pl(r) = numel(find(pl_mask(px) > 0.07))./numel(px);
         % region center coordinates
         [xs ys] = ind2sub([sx sy], px);
         reg.center_x(r) = mean(xs);
         reg.center_y(r) = mean(ys);
      end
      % compute region comparison features
      for r = 1:n_reg
         % get neighboring regions
         r_nbs = sdata.seg_arr{lvl}.graph{r};
         % compare to neighbors
         for n = 1:numel(r_nbs)
            r2 = r_nbs(n);
            reg.fg_diff_avg(r) = ...
               reg.fg_diff_avg(r) + (reg.fg_avg(r) - reg.fg_avg(r2));
            reg.pl_diff_avg(r) = ...
               reg.pl_diff_avg(r) + (reg.pl_avg(r) - reg.pl_avg(r2));
            reg.fg_diff_max(r) = ...
               max(reg.fg_diff_max(r), (reg.fg_avg(r) - reg.fg_avg(r2)));
            reg.pl_diff_max(r) = ...
               max(reg.pl_diff_max(r), (reg.pl_avg(r) - reg.pl_avg(r2)));
         end
         reg.fg_diff_avg(r) = reg.fg_diff_avg(r) ./ (numel(r_nbs) + eps);
         reg.pl_diff_avg(r) = reg.pl_diff_avg(r) ./ (numel(r_nbs) + eps);
      end
      % project features onto base level regions
      sfeat.f.seg_size(:,lvl)    = reg.seg_size(sdata.id_map_arr{lvl});
      sfeat.f.fg_avg(:,lvl)      = reg.fg_avg(sdata.id_map_arr{lvl});
      sfeat.f.fg_norm_avg(:,lvl) = reg.fg_norm_avg(sdata.id_map_arr{lvl});
      sfeat.f.fg_diff_avg(:,lvl) = reg.fg_diff_avg(sdata.id_map_arr{lvl});
      sfeat.f.fg_diff_max(:,lvl) = reg.fg_diff_max(sdata.id_map_arr{lvl});
      sfeat.f.pl_avg(:,lvl)      = reg.pl_avg(sdata.id_map_arr{lvl});
      sfeat.f.pl_diff_avg(:,lvl) = reg.pl_diff_avg(sdata.id_map_arr{lvl});
      sfeat.f.pl_diff_max(:,lvl) = reg.pl_diff_max(sdata.id_map_arr{lvl});
      sfeat.f.cov_fg(:,lvl)      = reg.cov_fg(sdata.id_map_arr{lvl});
      sfeat.f.cov_pl(:,lvl)      = reg.cov_pl(sdata.id_map_arr{lvl});
      % compute features with respect to bounding box at base level
      rxs = reg.center_x(sdata.id_map_arr{lvl});
      rys = reg.center_y(sdata.id_map_arr{lvl});
      for r = 1:n_reg_base
         % check for associated object detection
         if (sfeat.obj_id(r) > 0)
            % get detection bounding box
            bounds = ...
               round(double(scene.dets.obj_bounds.bounds(sfeat.obj_id(r),:)));
            ys = bounds(1); ylen = bounds(3); ye = ys+ylen-1;
            xs = bounds(2); xlen = bounds(4); xe = xs+xlen-1;
            % normalize segment size by bounding box size
            sfeat.f.frac_size(r,lvl) = sfeat.f.seg_size(r,lvl) ./ (xlen.*ylen);
            % compute bounding box widths
            xw = xlen./2;
            yw = ylen./2;
            % compute bounding box reference points
            xc  = (xs + xe)./2;  yc  = (ys + ye)./2;
            xur = xc + xw;       yur = yc + yw;
            xul = xc - xw;       yul = yc + yw;
            xlr = xc + xw;       ylr = yc - yw;
            xll = xc - xw;       yll = yc - yw;
            % get segment center
            rx = rxs(r);
            ry = rys(r);
            % compute distances to bounding box reference points
            dc  = sqrt((rx - xc).^2  + (ry - yc).^2);
            dur = sqrt((rx - xur).^2 + (ry - yur).^2);
            dul = sqrt((rx - xul).^2 + (ry - yul).^2);
            dlr = sqrt((rx - xlr).^2 + (ry - ylr).^2);
            dll = sqrt((rx - xll).^2 + (ry - yll).^2);
            dx = abs(rx - xc);
            dy = abs(ry - yc);
            % translate distances into affinities
            sigma = sqrt(xw.*xw + yw.*yw);
            sfeat.f.aff_obj_center(r,lvl) = exp(-dc./sigma);
            sfeat.f.aff_obj_ur(r,lvl) = exp(-dur./sigma);
            sfeat.f.aff_obj_ul(r,lvl) = exp(-dul./sigma);
            sfeat.f.aff_obj_lr(r,lvl) = exp(-dlr./sigma);
            sfeat.f.aff_obj_ll(r,lvl) = exp(-dll./sigma);
            sfeat.f.aff_obj_x(r,lvl)  = exp(-dx./xw);
            sfeat.f.aff_obj_y(r,lvl)  = exp(-dy./yw);
         else
            % zero fractional segment size (no associated bounding box)
            sfeat.f.frac_size(r,lvl) = 0;
            % zero affinities to bounding box reference points
            sfeat.f.aff_obj_center(r,lvl) = 0;
            sfeat.f.aff_obj_ur(r,lvl) = 0;
            sfeat.f.aff_obj_ul(r,lvl) = 0;
            sfeat.f.aff_obj_lr(r,lvl) = 0;
            sfeat.f.aff_obj_ll(r,lvl) = 0;
            sfeat.f.aff_obj_x(r,lvl) = 0;
            sfeat.f.aff_obj_y(r,lvl) = 0;
         end
      end
   end
end
