% Run batch jobs for pb, poselets, and globalized scene interpretation.
%
% run_batch(images, [options], [machine_list], [n_jobs])
%
% options = run_batch('options') % return default options
%
% Input:
%    images.            - images to process
%       im_list            - image file names or {} for all in source directory
%       dir_im             - source image directory
%       dir_out            - data output directory
%    options.           - processing and output options
%       recompute_pb       - force recomputation of pb? (default: false)
%       recompute_poselets - force recomputation of poselets? (default: false)
%       recompute_scene    - force recomputation of scene? (default: false)
%       recompute_vis      - force recreation of visualization? (default: false)
%       recompute_mask     - force recreation of object masks? (default: false)
%       recompute_voc_det  - force recreation of VOC detection results?
%       params_scene       - globalization parameters (see: seg_obj)
%       tag                - tag associated to output (default: '')
%    machine_list       - machines on which to execute cluster jobs
%                         (if [] then only use current machine)
%    n_jobs             - max number of jobs to launch in parallel (default: 1)
function options = run_batch(images, options, machine_list, n_jobs)
   % default options
   options_def.recompute_pb       = false;
   options_def.recompute_poselets = false;
   options_def.recompute_scene    = false;
   options_def.recompute_vis      = false;
   options_def.recompute_mask     = false;
   options_def.recompute_voc_det  = false;
   options_def.params_scene       = seg_obj('params');
   options_def.tag                = '';
   % check if returning default options requested
   if ((nargin == 1) && (ischar(images)))
      % return default options
      options = options_def;
      return;
   end
   % check arguments
   if ((nargin < 2) || isempty(options)), options = options_def; end
   if ((nargin < 3)), machine_list = []; end
   if ((nargin < 4) || isempty(n_jobs)), n_jobs = 1; end
   % set unspecified options to defaults
   options = set_defaults(options, options_def);
   % convert input/output directories to absolute path form
   images.dir_im  = absolute_path(images.dir_im);
   images.dir_out = absolute_path(images.dir_out);
   % if requested, scan input directory for image files
   if (isempty(images.im_list))
      im_files = dir(images.dir_im);
      is_im = ~([im_files.isdir]);
      im_files = {im_files.name};
      im_files = im_files(is_im);
   else
      im_files = images.im_list;
   end
   % compute names of output directories
   if (isempty(options.tag))
      dir_results = fullfile(images.dir_out,'results');
   else
      dir_results = fullfile(images.dir_out,['results_' options.tag]);
   end
   dir_pb       = fullfile(images.dir_out,'pb');
   dir_poselets = fullfile(images.dir_out,'poselets');
   dir_scene    = fullfile(dir_results,'scene');
   dir_vis      = fullfile(dir_results,'vis');
   dir_vis_all  = fullfile(dir_results,'vis_all');
   dir_mask     = fullfile(dir_results,'mask');
   dir_mask_vis    = fullfile(dir_mask,'vis');
   dir_mask_pl     = fullfile(dir_mask,'pl');
   dir_mask_top    = fullfile(dir_mask,'top');
   dir_mask_top_fg = fullfile(dir_mask,'top_fg');
   dir_voc_pl  = fullfile(dir_results,'voc_det_pl');
   dir_voc_sc  = fullfile(dir_results,'voc_det_sc');
   % create output directories if they don't exist
   if (~exist(dir_pb,'dir')),       mkdir(dir_pb);       end
   if (~exist(dir_poselets,'dir')), mkdir(dir_poselets); end
   if (~exist(dir_results,'dir')),  mkdir(dir_results);  end
   if (~exist(dir_scene,'dir')),    mkdir(dir_scene);    end
   if (~exist(dir_vis,'dir')),      mkdir(dir_vis);      end
   if (~exist(dir_vis_all,'dir')),  mkdir(dir_vis_all);  end
   if (~exist(dir_mask,'dir')),     mkdir(dir_mask);     end
   if (~exist(dir_mask_vis,'dir')),    mkdir(dir_mask_vis);    end
   if (~exist(dir_mask_pl,'dir')),     mkdir(dir_mask_pl);     end
   if (~exist(dir_mask_top,'dir')),    mkdir(dir_mask_top);    end
   if (~exist(dir_mask_top_fg,'dir')), mkdir(dir_mask_top_fg); end
   if (~exist(dir_voc_pl,'dir')), mkdir(dir_voc_pl); end
   if (~exist(dir_voc_sc,'dir')), mkdir(dir_voc_sc); end
   % save options to file
   file_opt = fullfile(dir_results,'options.mat');
   save(file_opt,'options');
   % assemble job list
   n_images = numel(im_files);
   jobs = cell([1 n_images]);
   for n = 1:n_images
      % get image filename parts
      [pathstr name extn] = fileparts(im_files{n});
      % assemble job file io names
      io.name          = name;
      io.file_opt      = file_opt;
      io.file_im       = fullfile(images.dir_im, im_files{n});
      io.file_pb       = fullfile(dir_pb, [name '-pb.mat']);
      io.file_poselets = fullfile(dir_poselets, [name '-poselets.mat']);
      io.file_scene    = fullfile(dir_scene, [name '-scene.mat']);
      io.dir_vis       = fullfile(dir_vis, name);
      io.dir_vis_all   = dir_vis_all;
      io.dir_mask      = dir_mask;
      io.file_voc_pl   = fullfile(dir_voc_pl, [name '.txt']);
      io.file_voc_sc   = fullfile(dir_voc_sc, [name '.txt']);
      io.vis_link      = ['ln -s ' '../vis/' name '/all.png ' name '_all.png'];
      io.file_error    = fullfile(dir_results, [name '-error.txt']);
      % store job arguments
      jobs{n} = { io };
   end
   %  local or cluster execution parameters
   if (isempty(machine_list))
      p_distr = {'type','local'};
   else
      p_distr = {'type','distr','pLaunch',{n_jobs+2,machine_list}};
   end
   % run jobs
   if (~isempty(jobs))
      fevalDistr('run_batch_job',jobs,p_distr{:});
   end
end
