% Run a job to compute pb, poselets, and scene results for an image.
%
% flag = run_batch_job(io)
%
% Input:
%    io.                - specification of files on which to operate
%       name               - base filename of image
%       file_opt           - file storing computation options
%       file_im            - input image file
%       file_pb            - output pb file
%       file_poselets      - output poselets file
%       file_scene         - output scene file
%       dir_vis            - output visualization directory
%       dir_vis_all        - output visualization summary directory
%       dir_mask           - output mask directory
%       file_voc_pl        - VOC poselet detection output file 
%       file_voc_sc        - VOC scene detection output file 
%       vis_link           - linking command for summary output
%       file_error         - file in which to log any errors
%
% Output:
%       flag            - true on successful completion
function flag = run_batch_job(io)
   flag = true;
   try
      % load options
      options = load(io.file_opt,'options');
      options = options.options;
      % check whether to compute pb
      if ((options.recompute_pb) || (~exist(io.file_pb,'file')))
         % read image
         im = double(imread(io.file_im))./255;
         % compute pb
         pb = compute_pb(im);
         % save pb
         pb_save(pb, io.file_pb);
      end
      % check whether to compute poselets
      if ((options.recompute_poselets) || (~exist(io.file_poselets,'file')))
         % read image
         im_u8 = imread(io.file_im);
         % compute poselets
         p_dets = compute_poselets(im_u8);
         % save poselets
         save(io.file_poselets,'p_dets');
      end
      % check whether to compute scene globalization
      if ((options.recompute_scene) || (~exist(io.file_scene,'file')))
         % load pb
         pb = pb_load(io.file_pb);
         % load poselets
         p_dets = load(io.file_poselets,'p_dets');
         p_dets = p_dets.p_dets;
         % compute scene globalization
         scene = seg_obj(pb, p_dets.poselet_hits, options.params_scene);
         % save scene results
         save(io.file_scene,'scene');
      end
      % check whether to create visualization
      if ((options.recompute_vis) || (~exist(io.dir_vis,'dir')))
         % read image
         im = double(imread(io.file_im))./255;
         % load scene results
         scene = load(io.file_scene,'scene');
         scene = scene.scene;
         % compute visualization
         vis = vis_generate(im, scene);
         % create output
         vis_write(vis, io.dir_vis);
         % link summary visualization
         wd = cd;
         cd(io.dir_vis_all);
         system(io.vis_link);
         cd(wd);
      end
      % check whether to create masks
      if ((options.recompute_mask) || (~exist(io.dir_mask,'dir')))
         % read image
         im = double(imread(io.file_im))./255;
         % load poselets
         p_dets = load(io.file_poselets,'p_dets');
         p_dets = p_dets.p_dets;
         % load scene results
         scene = load(io.file_scene,'scene');
         scene = scene.scene;
         % write masks
         obj_mask_create( ...
            im, p_dets, scene, options.params_scene, io.dir_mask, io.name);
      end
      % check whether to create VOC detection output
      if ((options.recompute_voc_det) || ...
          (~exist(io.file_voc_pl,'file')) || ...
          (~exist(io.file_voc_sc,'file')))
         % read image
         im = double(imread(io.file_im))./255;
         % load poselets
         p_dets = load(io.file_poselets,'p_dets');
         p_dets = p_dets.p_dets;
         % load scene results
         scene = load(io.file_scene,'scene');
         scene = scene.scene;
         % write VOC detection results
         [sx sy sz] = size(im);
         voc_det_extract( ...
            io.file_voc_pl, io.name, p_dets.bounds_predictions, [sx sy]);
         voc_det_extract( ...
            io.file_voc_sc, io.name, scene.dets.obj_bounds, [sx sy]);
      end
      % check whether to create refined output
      if (~exist([io.file_scene(1:end-4) '-refined.mat'],'file'))
         % load pb
         pb = pb_load(io.file_pb);
         % load scene
         scene = load(io.file_scene,'scene');
         scene = scene.scene;
         % select poselets in verified detections
         poselet_select = zeros([1 scene.dets.poselet_hits.size]);
         for d = 1:(scene.dets.obj_bounds.size)
            if (scene.dets.obj_bounds.score(d) > ...
                options.params_scene.det_score_th)
               poselet_select(scene.dets.obj_bounds.src_idx{d}) = 1;
            end
         end
         poselet_inds = find(poselet_select);
         poselet_hits = scene.dets.poselet_hits.select(poselet_inds);
         % recompute scene globalization
         scene_refined = seg_obj(pb, poselet_hits, options.params_scene);
         % save refined scene results
         save([io.file_scene(1:end-4) '-refined.mat'],'scene_refined');
      end
   catch
      flag = false;
      try
         % retrieve error message
         err = lasterror;
         % log error
         f = fopen(io.file_error,'w');
         fprintf(f,'%s\n',err.message);
         fclose(f);
      catch
      end
   end
end
